#! /bin/csh -f
#
#	script for viewing a Patterson map in O
#
#
#
# set up awk
alias nawk /usr/bin/nawk
nawk 'BEGIN{print 1; exit}' >& /dev/null
if($status) alias nawk awk


set pattfile  = FH_Patt.map
set min_sig   = 2
set step      = 1

set MAPMAN    = /programs/o/rave/rave_irix6/6d_mapman
set O         = /programs/o/bin/sg_sauron_6.2
setenv ODAT     /programs/o/data

set tempfile = opatt


###########################################################################
if($#argv == 0) goto Help
goto Setup
# scan the command line for input
Help:
###########################################################################
cat << EOF

usage: $0 $pattfile mapman o

where: 
$pattfile is the CCP4 Patterson you want to view in O
mapman    (optional) is the location of the RAVE mapman program 
o         (optional) is the location of the O executable

EOF
rm -f ${tempfile}* >& /dev/null

exit 9
ReturnFromSetup:
###########################################################################
# the "Setup" prodecure should set the values of:
# $pattfile $MAPMAN $O
# from the command line, the defaults, or sensible deductions


echo "scale sigma" |\
mapmask mapin $pattfile mapout ${tempfile}.map

# peak-pick (for point-and-click map intensity)
peakmax mapin ${tempfile}.map xyzout ${tempfile}.pdb << EOF
THRESHOLD RMS 2
EOF

# make a grid for finding Harker sections
#nawk 'BEGIN{for(x=0;x<=1;x+=0.1){\
#            for(y=0;y<=1;y+=0.1){\
#            for(z=0;z<=1;z+=0.1){\
#            print "ATOM X", x, y, z}}}}' |\
#cat >! ${tempfile}atoms



# convert the Patterson to O format
setenv MAPSIZE `ls -l ${tempfile}.map | nawk '{print $5/3.9}'`
$MAPMAN << end-mapman |& grep -v Toodle
read map1 ${tempfile}.map CCP4
mappage map1 ${tempfile}.omap
quit
y
end-mapman
rm -f ${tempfile}.map >& /dev/null

# make a nice O macro for viewing it
cat << EOF >! ${tempfile}.omac







sam_atom_in ${tempfile}.pdb peaks
mol peaks
obj peaks
zone ;
end
sym_set ; ; ; ;
sym_cell
paint_col black
paint_object peaks

map_cache
map_active_center
map_file ${tempfile}.omap
EOF

# start 1 sigma contours at 2nd highest peak
cat ${tempfile}.pdb | nawk '{printf "%d\n", substr($0, 61, 6)}' |\
sort -unr | nawk 'NR==2' |\
nawk -v min_sig=$min_sig -v step=$step '{top=$1; \
  for(sig=top;sig>=min_sig;sig-=step){if(min_sig==top)top=min_sig+1;\
    x=(sig-min_sig+1)/(top-min_sig+1)\
    r=(x)^3;\
    g=x*cos(4.8*(1-x))^2;\
    b=(x);\
    print "map_object pat" sig;\
    printf "map_param 500 500 500 %d %.2f %.2f %.2f 0.5 0 1\n", sig, r, g, b;\
    print "map_draw"}}' |\
cat >> ${tempfile}.omac

# launch O, redirecting stdin from here
cat ${tempfile}.omac - | $O
if($status) then
    echo "to view ${pattfile},"
    echo "run ${tempfile}.omac in o"
    exit
endif

# clean up
rm -f ${tempfile}.omac >& /dev/null
rm -f ${tempfile}.omap >& /dev/null
rm -f ${tempfile}.pdb  >& /dev/null

exit
###########################################################################


###########################################################################
###########################################################################
Setup:
set MAXLINE = 1000

foreach arg ( $* )

    # recognize map files
    if("$arg" =~ *.map) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	
	set pattfile = "$arg"
	continue
    endif
    
    # is it mapman?
    if(("$arg" =~ *mapman*)||("$arg" !~ *xdlmapman*)) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	echo "testing $arg"
	set temp = `setenv DISPLAY; echo "quit" | "$arg" |& nawk '/Kleywegt/{print}' | tail -1`
	if("$temp" != "") then
	    set MAPMAN = "$arg"
	    set file = ""
	endif
    endif
end    

if(! -e "$pattfile") goto Help

########################################################################################################
#
#	find MAPMAN 
#
########################################################################################################
set program  = "$MAPMAN"
set names    = "mapman MAPMAN 6d_mapman 4d_mapman"
set badname  = "xdlmapman"
set places = "~/bin /programs/bin /programs /xtal /usr/local/bin /usr/local /usr/xtal /usr"

set size        = 100000
set program_Version = "0"
set GoodVersion     = "4"

# make sure no xwindows program pop up
if($?DISPLAY) then
    set noDISPLAY = $DISPLAY
    setenv DISPLAY
endif

# first, check to see if default will work
test -x "$program"
if(! $status) then
    set file = `ls -lL $program |& nawk -v size=$size '$5>size{print $NF}'`
    # test for program signature (author and version number)
    set temp = `echo "" | $file |& nawk '/Kleywegt/ && name{print ver, 1;exit} /MAPMAN/{name=1} /Version/{split($NF,w,"/"); ver=w[2]}'`
    if("$temp" != "") then
	# get most recent version (if possible)
	set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
	if("$temp" != "") then
	    set program = "$file"
	    set program_Version = "$temp"
	endif
    endif
endif


# try using the "which" command
foreach name ( $names )
    # check for sufficiently high version number
    set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
    if("$temp" != "") break
    
    # "which" should handle processing of $path
    set possibilities = `which $name |& nawk 'NF==1{gsub(/[*?\042\047]/," "); print}' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print}'`
    foreach file ( $possibilities )
	# skip filenames we know are wrong
	if("$file" =~ *$badname) continue
	
	# check to see if it's even executable
	test -x "$file"
	if(! $status) then
	    # test for program signature (author and version number)
	    set temp = `echo "" | $file |& nawk '/Kleywegt/ && name{print ver, 1;exit} /MAPMAN/{name=1} /Version/{split($NF,w,"/"); ver=w[2]}'`
	    if("$temp" != "") then
		# get most recent version (if possible)
		set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		if("$temp" != "") then
		    set program = "$file"
		    set program_Version = "$temp"
		endif
	    endif
	endif
    end
end


# check again for sufficiently high version number
set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
if("$temp" == "") then    

    # simplefy program name
    set program = `echo $names | nawk '{print $1}'`
    
    # search for $program in likely places
    echo -n "Looking for $program "
    set HARD_TO_FIND
    foreach place ( $places )
	if(-e $place) then
	    # keep looking
	    set files = `ls -lL ${place} |& grep "$program" |& sort -nr +4 |& nawk -v size=$size ' /^-/ && $1 ~ /x/ && $5 > size {print $NF}' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print}'`
	    foreach file ( $files )
		# skip filenames we know are wrong
		if("$file" =~ *$badname) continue
		
		set file = "${place}/${file}"
		# test for program signature (author and version number)
		set temp = `echo "" | $file |& nawk '/Kleywegt/ && name{print ver, 1;exit} /MAPMAN/{name=1} /Version/{split($NF,w,"/"); ver=w[2]}'`
		if("$temp" != "") then
		    # get most recent version (if possible)
		    set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		    if("$temp" != "") then
			set program = "$file"
			set program_Version = "$temp"
		    endif
		endif
	    end
	endif
		
	# entertainment
	echo -n "."

	# check for sufficiently high version number
	set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
	if("$temp" != "") break
    end
endif


# check again for sufficiently high version number
set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
if("$temp" == "") then

    # get absolute time (seconds since 1/1/1)
    set THEN = `date "+%S %M %H %j %Y" |& nawk '{$5-=1; printf "%50.0f\n", $1+ 60*($2+ 60*($3+ 24*($4 + 365 *$5 + int($5/4) -int($5/100) +int($5/400) )))}'`

    echo -n " looking harder "
    onintr Skip_MAPMAN_search
    
    foreach place ( $places )
	# check for sufficiently high version number
	set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
	if("$temp" != "") break
	
	if(-e $place) then
	    if("$place" == "/") echo -n "hmm."
	
	    # use find to get candidate files
	    set files = `find $place -name '*'$program'*' \( -type l -o \( -type f -size +${size}c \) \) -perm -1 -print |& nawk '! /^find:/' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print} l>L{exit}'`
	    foreach file ( $files )
		# skip filenames we know are wrong
		if("$file" =~ *$badname) continue

		# test for program signature (author and version number)
		set temp = `echo "" | $file |& nawk '/Kleywegt/ && name{print ver, 1;exit} /MAPMAN/{name=1} /Version/{split($NF,w,"/"); ver=w[2]}'`
		if("$temp" != "") then
		    # get most recent version (if possible)
		    set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		    if("$temp" != "") then
			set program = "$file"
			set program_Version = "$temp"
		    endif
		endif
	    end
	endif
	
	# entertainment
	echo -n "."

	# give up if this is taking too long (10 minutes)
	set NOW = `date "+%S %M %H %j %Y" |& nawk '{$5-=1; printf "%50.0f\n", $1+ 60*($2+ 60*($3+ 24*($4 + 365 *$5 + int($5/4) -int($5/100) +int($5/400) )))}'`
	set temp = `echo "$NOW $THEN" | nawk '{printf "%d", $1 - $2}'`
	if($temp > 600) then
	    echo " this is taking too long."
	    break;
	endif	
    end
endif

Skip_MAPMAN_search:
onintr

# check if we found any program
test -x "$program"
if(! $status) then
    # got something, make sure it works
    set file = $program
    set temp = `echo "" | $file |& nawk '/Kleywegt/ && name{print ver, 1;exit} /MAPMAN/{name=1} /Version/{split($NF,w,"/"); ver=w[2]}'`
    if("$temp" != "") then
	set MAPMAN = $program
	set MAPMAN_Version = "$temp"
    else
	# this shouldn't happen, but...
#	set MAPMAN = ""
    endif
else
#    set MAPMAN = ""
endif

# turn display back on
if($?noDISPLAY) then
    setenv DISPLAY $noDISPLAY
    unset noDISPLAY
endif

# out of ideas, help?
test -x "$MAPMAN"
if(! $status) then
    echo ""
    if($?HARD_TO_FIND) echo "found $MAPMAN"
else
    echo ""
    echo "Couldn't find ${program}, looked everywhere."
    set MAPMAN_Version = "0"
    set NO_MAPMAN
    goto Help
endif
unset HARD_TO_FIND



########################################################################################################
#
#	find O
#
########################################################################################################
set program  = "$O"
set names    = "ono 4d_ono 4d_impact sg_ono sg_sauron"
set places = "~/bin /programs/bin /programs /xtal /usr/local/bin /usr/local /usr/xtal /usr"

set size        = 100000
set program_Version = "0"
set GoodVersion     = "5"

# make sure no xwindows program pops up
if($?DISPLAY) then
    set noDISPLAY = $DISPLAY
#    setenv DISPLAY
endif

# first, check to see if default will work
test -x "$program"
if(! $status) then
    set file = `ls -lL $program |& nawk -v size=$size '$5>size{print $NF}'`
    # test for program signature (author and version number)
    set temp = `echo "quit" | $file |& nawk '/ O version/{print $5;exit} NR>1000{exit}'`
    if("$temp" != "") then
	# get most recent version (if possible)
	set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
	if("$temp" != "") then
	    set program = "$file"
	    set program_Version = "$temp"
	endif
    endif
endif


# try using the "which" command
foreach name ( $names )
    # check for sufficiently high version number
    set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
    if("$temp" != "") break
    
    # "which" should handle processing of $path
    set possibilities = `which $name |& nawk 'NF==1{gsub(/[*?\042\047]/," "); print}' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print}'`
    foreach file ( $possibilities )
	# skip filenames we know are wrong
	if("$file" =~ *$badname) continue
	
	# check to see if it's even executable
	test -x "$file"
	if(! $status) then
	    # test for program signature (author and version number)
	    set temp = `echo "quit" | $file |& nawk '/ O version/{print $5;exit} NR>1000{exit}'`
	    if("$temp" != "") then
		# get most recent version (if possible)
		set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		if("$temp" != "") then
		    set program = "$file"
		    set program_Version = "$temp"
		endif
	    endif
	endif
    end
end


# check again for sufficiently high version number
set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
if("$temp" == "") then    

    # simplefy program name
    set program = `echo $names | nawk '{print $1}'`
    
    # search for $program in likely places
    echo -n "Looking for $program "
    set HARD_TO_FIND
    foreach place ( $places )
	if(-e $place) then
	    # keep looking
	    set files = `ls -lL ${place} |& grep "$program" |& sort -nr +4 |& nawk -v size=$size ' /^-/ && $1 ~ /x/ && $5 > size {print $NF}' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print}'`
	    foreach file ( $files )
		# skip filenames we know are wrong
		if("$file" =~ *$badname) continue
		
		set file = "${place}/${file}"
		# test for program signature (author and version number)
		set temp = `echo "quit" | $file |& nawk '/ O version/{print $5;exit} NR>1000{exit}'`
		if("$temp" != "") then
		    # get most recent version (if possible)
		    set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		    if("$temp" != "") then
			set program = "$file"
			set program_Version = "$temp"
		    endif
		endif
	    end
	endif
		
	# entertainment
	echo -n "."

	# check for sufficiently high version number
	set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
	if("$temp" != "") break
    end
endif


# check again for sufficiently high version number
set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
if("$temp" == "") then

    # get absolute time (seconds since 1/1/1)
    set THEN = `date "+%S %M %H %j %Y" |& nawk '{$5-=1; printf "%50.0f\n", $1+ 60*($2+ 60*($3+ 24*($4 + 365 *$5 + int($5/4) -int($5/100) +int($5/400) )))}'`

    echo -n " looking harder "
    onintr Skip_O_search
    
    foreach place ( $places )
	# check for sufficiently high version number
	set temp = `echo "$program_Version $GoodVersion" | nawk '$1+0 >= $2+0{print $1}'`
	if("$temp" != "") break
	
	if(-e $place) then
	    if("$place" == "/") echo -n "hmm."
	
	    # use find to get candidate files
	    set files = `find $place -name '*'$program'*' \( -type l -o \( -type f -size +${size}c \) \) -perm -1 -print |& nawk '! /^find:/' |& nawk -v L=$MAXLINE '{l+=length($0)} l<L{print} l>L{exit}'`
	    foreach file ( $files )
		# skip filenames we know are wrong
		if("$file" =~ *$badname) continue

		# test for program signature (author and version number)
		set temp = `echo "quit" | $file |& nawk '/ O version/{print $5;exit} NR>1000{exit}'`
		if("$temp" != "") then
		    # get most recent version (if possible)
		    set temp = `echo $temp $program_Version | nawk '$1+0 > $2+0{print $1}'`
		    if("$temp" != "") then
			set program = "$file"
			set program_Version = "$temp"
		    endif
		endif
	    end
	endif
	
	# entertainment
	echo -n "."

	# give up if this is taking too long (10 minutes)
	set NOW = `date "+%S %M %H %j %Y" |& nawk '{$5-=1; printf "%50.0f\n", $1+ 60*($2+ 60*($3+ 24*($4 + 365 *$5 + int($5/4) -int($5/100) +int($5/400) )))}'`
	set temp = `echo "$NOW $THEN" | nawk '{printf "%d", $1 - $2}'`
	if($temp > 600) then
	    echo " this is taking too long."
	    break;
	endif	
    end
endif

Skip_O_search:
onintr

# check if we found any program
test -x "$program"
if(! $status) then
    # got something, make sure it works
    set file = $program
    set temp = `echo "quit" | $file |& nawk '/ O version/{print $5;exit} NR>1000{exit}'`
    if("$temp" != "") then
	set O = $program
	set O_Version = "$temp"
    else
	# this shouldn't happen, but...
#	set O = ""
    endif
else
#    set O = ""
endif

# turn display back on
if($?noDISPLAY) then
    setenv DISPLAY $noDISPLAY
endif
unset noDISPLAY


# out of ideas, help?
test -x "$O"
if(! $status) then
    echo ""
    if($?HARD_TO_FIND) echo "found $O"
else
    echo ""
    echo "Couldn't find ${program}, looked everywhere."
    set O_Version = "0"
    set NO_O
endif
unset HARD_TO_FIND


# set up O data directory
if(! -e "$ODAT") then
    set ODAT = "$O"
    while(("$ODAT" != ".")&&("$ODAT" != "/"))
	set ODAT = `dirname $ODAT`
	set temp = `find $ODAT -name 'menu.o' -print |& nawk 'NF==1{print; exit}'`
	if("$temp" != "") then
	    set ODAT = `dirname $temp`
	    break
	endif
    end
endif



goto ReturnFromSetup
